@@ -1,10 +1,10 @@
 /***********************************************************************
  field_names.cpp - Implements the FieldNames class.
 
- Copyright (c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by
- MySQL AB, and (c) 2004, 2005 by Educational Technology Resources, Inc.
- Others may also hold copyrights on code in this file.  See the CREDITS
- file in the top directory of the distribution for details.
+ Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
+ (c) 2004-2007 by Educational Technology Resources, Inc.  Others may
+ also hold copyrights on code in this file.  See the CREDITS file in
+ the top directory of the distribution for details.
 
  This file is part of MySQL++.
 
@@ -28,21 +28,32 @@
 #include "common.h"
 
 #include "field_names.h"
-
 #include "result.h"
 
+#include <algorithm>
+
 namespace mysqlpp {
 
-void FieldNames::init(const ResUse * res)
+void 
+FieldNames::init(const ResultBase* res)
 {
 	int num = res->num_fields();
 	reserve(num);
 
 	for (int i = 0; i < num; i++) {
-		std::string p(res->fields().at(i).name);
+		std::string p(res->fields().at(i).name());
 		str_to_lwr(p);
 		push_back(p);
 	}
 }
 
+
+unsigned int
+FieldNames::operator [](const std::string& s) const
+{
+	std::string temp(s);
+	str_to_lwr(temp);
+	return static_cast<unsigned int>(std::find(begin(), end(), temp) - begin());
+}
+
 } // end namespace mysqlpp
