@@ -2,10 +2,10 @@
 /// \brief Declares the classes for holding row data from a result set.
 
 /***********************************************************************
- Copyright (c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by
- MySQL AB, and (c) 2004, 2005 by Educational Technology Resources, Inc.
- Others may also hold copyrights on code in this file.  See the CREDITS
- file in the top directory of the distribution for details.
+ Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
+ (c) 2004-2007 by Educational Technology Resources, Inc.  Others may
+ also hold copyrights on code in this file.  See the CREDITS file in
+ the top directory of the distribution for details.
 
  This file is part of MySQL++.
 
@@ -25,162 +25,336 @@
  USA
 ***********************************************************************/
 
-#ifndef MYSQLPP_ROW_H
+#if !defined(MYSQLPP_ROW_H)
 #define MYSQLPP_ROW_H
 
-#include "coldata.h"
-#include "exceptions.h"
+#include "common.h"
+
+#include "mystring.h"
 #include "noexceptions.h"
-#include "resiter.h"
+#include "refcounted.h"
 #include "vallist.h"
 
 #include <vector>
 #include <string>
 
-#include <string.h>
-
 namespace mysqlpp {
 
 #if !defined(DOXYGEN_IGNORE)
 // Make Doxygen ignore this
 class FieldNames;
-class MYSQLPP_EXPORT ResUse;
+class MYSQLPP_EXPORT ResultBase;
 #endif
 
 /// \brief Manages rows from a result set.
-class MYSQLPP_EXPORT Row :
-		public const_subscript_container<Row, ColData, const ColData>,
-		public OptionalExceptions
+///
+/// This class is like an extended version of a \c const \c std::vector
+/// of mysqlpp::String.  It adds stuff for populating the vector.  As
+/// for why it's \c const, what would it mean to modify a Row?  If we
+/// ever did support such semantics, it should probably actually modify
+/// the database.  We can't do that if we just derive from std::vector.
+///
+/// Not that we could derive from std::vector even if we wanted to:
+/// \c vector::operator[](size_type) would interfere with our
+/// \c operator[](const \c char*).  We can avoid this only by
+/// maintaining our own public inteface independent of that of
+/// \c vector.
+
+class MYSQLPP_EXPORT Row : public OptionalExceptions
 {
+private:
+	/// \brief Pointer to bool data member, for use by safe bool
+	/// conversion operator.
+	///
+	/// \see http://www.artima.com/cppsource/safebool.html
+    typedef bool Row::*private_bool_type;
+
 public:
+	/// \brief type of our internal data list
+	///
+	/// This is public because all other typedefs we have for
+	/// mirroring std::vector's public interface depend on it.
+	typedef std::vector<String> list_type;
+
+	/// \brief constant iterator type
+	typedef list_type::const_iterator const_iterator;
+
+	/// \brief constant reference type
+	typedef list_type::const_reference const_reference;
+
+	/// \brief const reverse iterator type
+	typedef list_type::const_reverse_iterator const_reverse_iterator;
+
+	/// \brief type for index differences
+	typedef list_type::difference_type difference_type;
+
+	/// \brief iterator type
+	///
+	/// Note that this is just an alias for the const iterator.  Row
+	/// is immutable, but people are in the habit of saying 'iterator'
+	/// even when they don't intend to use the iterator to modify the
+	/// container, so we provide this as a convenience.
+	typedef const_iterator iterator;
+
+	/// \brief reference type
+	///
+	/// \sa iterator for justification for this const_reference alias
+	typedef const_reference reference;
+
+	/// \brief mutable reverse iterator type
+	///
+	/// \sa iterator for justification for this const_reverse_iterator
+	/// alias
+	typedef const_reverse_iterator reverse_iterator;
+
+	/// \brief type of returned sizes
+	typedef list_type::size_type size_type;
+
+	/// \brief type of data in container
+	typedef list_type::value_type value_type;
+
 	/// \brief Default constructor
 	Row() :
-	res_(0),
 	initialized_(false)
 	{
 	}
 	
-	/// \brief Create a row object
-	///
-	/// \param d MySQL C API row data
-	/// \param r result set that the row comes from
-	/// \param jj length of each item in d
-	/// \param te if true, throw exceptions on errors
-	Row(const MYSQL_ROW& d, const ResUse* r,
-			unsigned long* jj, bool te = true);
-
-	/// \brief Destroy object
-	~Row();
-
-	/// \brief Get a reference to our parent class.
-	const ResUse& parent() const
+	/// \brief Copy constructor
+	Row(const Row& r) :
+	OptionalExceptions(),
+	data_(r.data_.begin(), r.data_.end()),
+	field_names_(r.field_names_),
+	initialized_(r.initialized_)
 	{
-		return *res_;
 	}
 
-	/// \brief Get the number of fields in the row.
-	size_type size() const;
+	/// \brief Create a row object
+	///
+	/// \param row MySQL C API row data
+	/// \param res result set that the row comes from
+	/// \param lengths length of each item in row
+	/// \param te if true, throw exceptions on errors
+	Row(MYSQL_ROW row, const ResultBase* res,
+			const unsigned long* lengths, bool te = true);
+
+	/// \brief Destroy object
+	~Row() { }
+
+	/// \brief Get a const reference to the field given its index
+	///
+	/// If the index value is bad, the underlying std::vector is
+	/// supposed to throw an exception, according to the Standard.
+	const_reference at(size_type i) const { return data_.at(i); }
+
+	/// \brief Get a reference to the last element of the vector
+	const_reference back() const { return data_.back(); }
+
+	/// \brief Return a const iterator pointing to first element in the
+	/// container
+	const_iterator begin() const { return data_.begin(); }
+
+	/// \brief Returns true if container is empty
+	bool empty() const { return data_.empty(); }
+
+	/// \brief Return a const iterator pointing to one past the last
+	/// element in the container
+	const_iterator end() const { return data_.end(); }
+
+	/// \brief Get an "equal list" of the fields and values in this row
+	///
+	/// When inserted into a C++ stream, the delimiter 'd' will be used
+	/// between the items, " = " is the relationship operator, and items
+	/// will be quoted and escaped.
+	equal_list_ba<FieldNames, Row, quote_type0>
+			equal_list(const char* d = ",", const char* e = " = ") const;
+
+	/// \brief Get an "equal list" of the fields and values in this row
+	///
+	/// This method's parameters govern how the returned list will
+	/// behave when you insert it into a C++ stream:
+	///
+	/// \param d delimiter to use between items
+	/// \param e the operator to use between elements
+	/// \param m the manipulator to use for each element
+	///
+	/// For example, if d is ",", e is " = ", and m is the quote
+	/// manipulator, then the field and value lists (a, b) (c, d'e)
+	/// will yield an equal list that gives the following when inserted
+	/// into a C++ stream:
+	///
+	/// \code
+	///   'a' = 'c', 'b' = 'd''e'
+	/// \endcode
+	///
+	/// Notice how the single quote was 'escaped' in the SQL way to
+	/// avoid a syntax error.
+	template <class Manip>
+	equal_list_ba<FieldNames, Row, Manip> equal_list(const char* d,
+			const char* e, Manip m) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// When inserted into a C++ stream, the delimiter 'd' will be used
+	/// between the items, and no manipulator will be used on the items.
+	value_list_ba<FieldNames, do_nothing_type0>
+			field_list(const char* d = ",") const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// \param d delimiter to place between the items when the list is
+	/// inserted into a C++ stream
+	/// \param m manipulator to use before each item when the list is
+	/// inserted into a C++ stream
+	template <class Manip>
+	value_list_ba<FieldNames, Manip> field_list(const char* d,
+			Manip m) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// \param d delimiter to place between the items when the list is
+	/// inserted into a C++ stream
+	/// \param m manipulator to use before each item when the list is
+	/// inserted into a C++ stream
+	/// \param vb for each true item in this list, add that field name
+	/// to the returned list; ignore the others
+	template <class Manip>
+	value_list_b<FieldNames, Manip> field_list(const char* d, Manip m,
+			const std::vector<bool>& vb) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// \param d delimiter to place between the items when the list is
+	/// inserted into a C++ stream
+	/// \param vb for each true item in this list, add that field name
+	/// to the returned list; ignore the others
+	///
+	/// Field names will be quoted and escaped when inserted into a C++
+	/// stream.
+	value_list_b<FieldNames, quote_type0> field_list(
+			const char* d, const std::vector<bool>& vb) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// \param vb for each true item in this list, add that field name
+	/// to the returned list; ignore the others
+	///
+	/// Field names will be quoted and escaped when inserted into a C++
+	/// stream, and a comma will be placed between them as a delimiter.
+	value_list_b<FieldNames, quote_type0> field_list(
+			const std::vector<bool>& vb) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// For each true parameter, the field name in that position within
+	/// the row is added to the returned list.  When the list is
+	/// inserted into a C++ stream, the delimiter 'd' will be placed
+	/// between the items as a delimiter, and the manipulator 'm' used
+	/// before each item.
+	template <class Manip>
+	value_list_b<FieldNames, Manip> field_list(const char *d, Manip m,
+			bool t0,
+			bool t1 = false, bool t2 = false, bool t3 = false,
+			bool t4 = false, bool t5 = false, bool t6 = false,
+			bool t7 = false, bool t8 = false, bool t9 = false,
+			bool ta = false, bool tb = false, bool tc = false) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// For each true parameter, the field name in that position within
+	/// the row is added to the returned list.  When the list is
+	/// inserted into a C++ stream, the delimiter 'd' will be placed
+	/// between the items as a delimiter, and the items will be quoted
+	/// and escaped.
+	value_list_b<FieldNames, quote_type0> field_list(
+			const char *d, bool t0,
+			bool t1 = false, bool t2 = false, bool t3 = false,
+			bool t4 = false, bool t5 = false, bool t6 = false,
+			bool t7 = false, bool t8 = false, bool t9 = false,
+			bool ta = false, bool tb = false, bool tc = false) const;
+
+	/// \brief Get a list of the field names in this row
+	///
+	/// For each true parameter, the field name in that position within
+	/// the row is added to the returned list.  When the list is
+	/// inserted into a C++ stream, a comma will be placed between the
+	/// items as a delimiter, and the items will be quoted and escaped.
+	value_list_b<FieldNames, quote_type0> field_list(
+			bool t0,
+			bool t1 = false, bool t2 = false, bool t3 = false,
+			bool t4 = false, bool t5 = false, bool t6 = false,
+			bool t7 = false, bool t8 = false, bool t9 = false,
+			bool ta = false, bool tb = false, bool tc = false) const;
+
+	/// \brief Returns a field's index given its name
+	size_type field_num(const char* name) const;
+
+	/// \brief Get a reference to the first element of the vector
+	const_reference front() const { return data_.front(); }
+
+	/// \brief Return maximum number of elements that can be stored
+	/// in container without resizing.
+	size_type max_size() const { return data_.max_size(); }
+
+	/// \brief Assignment operator
+	Row& operator =(const Row& rhs)
+	{
+		data_.assign(rhs.data_.begin(), rhs.data_.end());
+		field_names_.assign(rhs.field_names_);
+		initialized_ = rhs.initialized_;
+		return *this;
+	}
 
 	/// \brief Get the value of a field given its name.
 	///
 	/// If the field does not exist in this row, we throw a BadFieldName
 	/// exception.
 	///
-	/// For this operator to work, the Result or ResUse object that
-	/// created this object must still exist.  In other words, you
-	/// cannot re-use or destroy the result object until you are done
-	/// retrieving data from this row object.
-	///
-	/// Note that we return the
-	/// \link mysqlpp::ColData_Tmpl ColData \endlink object by value.
-	/// The purpose of ColData is to make it easy to convert the string
-	/// data returned by the MySQL server to some more appropriate type,
-	/// so you're almost certain to use this operator in a construct
-	/// like this:
-	///
-	/// \code
-	///  string s = row["myfield"];
-	/// \endcode
-	///
-	/// That accesses myfield within the row, returns a temporary
-	/// ColData object, which is then automatically converted to a
-	/// \c std::string and copied into \c s.  That works fine, but
-	/// beware of this similar but incorrect construct:
-	///
-	/// \code
-	///  const char* pc = row["myfield"];
-	/// \endcode
-	///
-	/// This one line of code does what you expect, but \c pc is then a
-	/// dangling pointer: it points to memory owned by the temporary
-	/// ColData object, which will have been destroyed by the time you
-	/// get around to actually \e using the pointer.
-	///
-	/// This function is rather inefficient.  If that is a concern for
-	/// you, use at(), operator[](size_type) or the SSQLS mechanism'
-	/// instead.
-	const ColData operator [](const char* field) const;
+	/// This operator is fairly inefficient.  operator[](int) is faster.
+	const_reference operator [](const char* field) const;
 
 	/// \brief Get the value of a field given its index.
 	///
 	/// This function is just syntactic sugar, wrapping the at() method.
-	/// The at() method is the only way to get at the first field in a
-	/// result set by index, as \c row[0] is ambiguous: it could call
-	/// either \c operator[] overload.
 	///
-	/// \sa at() for the full documentation for this operator, and
-	/// operator[](const char*) for further caveats about using this
-	/// operator.
-	const ColData operator [](size_type i) const
+	/// It's \b critical that the parameter type be \c int, not
+	/// \c size_type, because it will interfere with the \c const
+	/// \c char* overload otherwise.  row[0] is ambiguous when there
+	/// isn't an int overload.
+	const_reference operator [](int i) const
+			{ return at(static_cast<size_type>(i)); }
+
+	/// \brief Returns true if row object was fully initialized and
+	/// has data.
+	///
+	/// This operator lets you use Row in bool context, which lets you
+	/// do things like tell when you've run off the end of a "use"
+	/// query's result set:
+	///
+	/// \code
+	///   Query q("....");
+	///   if (UseQueryResult res = q.use()) {
+	///       // Can use 'res', query succeeded
+	///       while (Row row = res.fetch_row()) {
+	///           // Retreived another row in the result set, can use 'row'
+	///       }
+	///   }
+	/// \endcode
+	///
+	operator private_bool_type() const
 	{
-		return at(i);
+		return data_.size() && initialized_ ? &Row::initialized_ : 0;
 	}
 
-	/// \brief Get the value of a field given its index.
-	///
-	/// If the index value is bad, the underlying std::vector is
-	/// supposed to throw an exception, according to the Standard.
-	///
-	/// For this function to work, the Result or ResUse object that
-	/// created this object must still exist.  In other words, you
-	/// cannot re-use or destroy the result object until you are done
-	/// retrieving data from this row object.
-	///
-	/// See operator[](const char*) for more caveats.
-	const ColData at(size_type i) const;
+	/// \brief Return reverse iterator pointing to first element in the
+	/// container
+	const_reverse_iterator rbegin() const { return data_.rbegin(); }
 
-	/// \brief Return the value of a field as a C string given its
-	/// index, in raw form.
-	///
-	/// This is the same thing as operator[], except that the data isn't
-	/// converted to a ColData object first.  Also, this method does not
-	/// check for out-of-bounds array indices.
-	const char* raw_data(int i) const
-	{
-		return data_[i].data();
-	}
+	/// \brief Return reverse iterator pointing to one past the last
+	/// element in the container
+	const_reverse_iterator rend() const { return data_.rend(); }
 
-	/// \brief Return the size of a field's raw data given its index.
-	std::string::size_type raw_size(int i) const
-	{
-		return data_[i].length();
-	}
-
-	/// \brief Return the value of a field as a C++ string given its
-	/// index, in raw form.
-	///
-	/// This is the same thing as operator[], except that the data isn't
-	/// converted to a ColData object first.
-	const std::string& raw_string(int i) const
-	{
-		return data_.at(i);
-	}
-
-	/// \brief Returns true if there is data in the row.
-	operator bool() const
-	{
-		return data_.size();
-	}
+	/// \brief Get the number of fields in the row.
+	size_type size() const { return data_.size(); }
 
 	/// \brief Get a list of the values in this row
 	///
@@ -342,138 +516,12 @@ public:
 		return value_list_b<Row, quote_type0>(*this, vb, ",", quote);
 	}
 
-	/// \brief Get a list of the field names in this row
-	///
-	/// When inserted into a C++ stream, the delimiter 'd' will be used
-	/// between the items, and no manipulator will be used on the items.
-	value_list_ba<FieldNames, do_nothing_type0>
-			field_list(const char* d = ",") const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// \param d delimiter to place between the items when the list is
-	/// inserted into a C++ stream
-	/// \param m manipulator to use before each item when the list is
-	/// inserted into a C++ stream
-	template <class Manip>
-	value_list_ba<FieldNames, Manip> field_list(const char* d,
-			Manip m) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// \param d delimiter to place between the items when the list is
-	/// inserted into a C++ stream
-	/// \param m manipulator to use before each item when the list is
-	/// inserted into a C++ stream
-	/// \param vb for each true item in this list, add that field name
-	/// to the returned list; ignore the others
-	template <class Manip>
-	value_list_b<FieldNames, Manip> field_list(const char* d, Manip m,
-			const std::vector<bool>& vb) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// \param d delimiter to place between the items when the list is
-	/// inserted into a C++ stream
-	/// \param vb for each true item in this list, add that field name
-	/// to the returned list; ignore the others
-	///
-	/// Field names will be quoted and escaped when inserted into a C++
-	/// stream.
-	value_list_b<FieldNames, quote_type0> field_list(
-			const char* d, const std::vector<bool>& vb) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// \param vb for each true item in this list, add that field name
-	/// to the returned list; ignore the others
-	///
-	/// Field names will be quoted and escaped when inserted into a C++
-	/// stream, and a comma will be placed between them as a delimiter.
-	value_list_b<FieldNames, quote_type0> field_list(
-			const std::vector<bool>& vb) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// For each true parameter, the field name in that position within
-	/// the row is added to the returned list.  When the list is
-	/// inserted into a C++ stream, the delimiter 'd' will be placed
-	/// between the items as a delimiter, and the manipulator 'm' used
-	/// before each item.
-	template <class Manip>
-	value_list_b<FieldNames, Manip> field_list(const char *d, Manip m,
-			bool t0,
-			bool t1 = false, bool t2 = false, bool t3 = false,
-			bool t4 = false, bool t5 = false, bool t6 = false,
-			bool t7 = false, bool t8 = false, bool t9 = false,
-			bool ta = false, bool tb = false, bool tc = false) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// For each true parameter, the field name in that position within
-	/// the row is added to the returned list.  When the list is
-	/// inserted into a C++ stream, the delimiter 'd' will be placed
-	/// between the items as a delimiter, and the items will be quoted
-	/// and escaped.
-	value_list_b<FieldNames, quote_type0> field_list(
-			const char *d, bool t0,
-			bool t1 = false, bool t2 = false, bool t3 = false,
-			bool t4 = false, bool t5 = false, bool t6 = false,
-			bool t7 = false, bool t8 = false, bool t9 = false,
-			bool ta = false, bool tb = false, bool tc = false) const;
-
-	/// \brief Get a list of the field names in this row
-	///
-	/// For each true parameter, the field name in that position within
-	/// the row is added to the returned list.  When the list is
-	/// inserted into a C++ stream, a comma will be placed between the
-	/// items as a delimiter, and the items will be quoted and escaped.
-	value_list_b<FieldNames, quote_type0> field_list(
-			bool t0,
-			bool t1 = false, bool t2 = false, bool t3 = false,
-			bool t4 = false, bool t5 = false, bool t6 = false,
-			bool t7 = false, bool t8 = false, bool t9 = false,
-			bool ta = false, bool tb = false, bool tc = false) const;
-
-	/// \brief Get an "equal list" of the fields and values in this row
-	///
-	/// When inserted into a C++ stream, the delimiter 'd' will be used
-	/// between the items, " = " is the relationship operator, and items
-	/// will be quoted and escaped.
-	equal_list_ba<FieldNames, Row, quote_type0>
-			equal_list(const char* d = ",", const char* e = " = ") const;
-
-	/// \brief Get an "equal list" of the fields and values in this row
-	///
-	/// This method's parameters govern how the returned list will
-	/// behave when you insert it into a C++ stream:
-	///
-	/// \param d delimiter to use between items
-	/// \param e the operator to use between elements
-	/// \param m the manipulator to use for each element
-	///
-	/// For example, if d is ",", e is " = ", and m is the quote
-	/// manipulator, then the field and value lists (a, b) (c, d'e)
-	/// will yield an equal list that gives the following when inserted
-	/// into a C++ stream:
-	///
-	/// \code
-	///   'a' = 'c', 'b' = 'd''e'
-	/// \endcode
-	///
-	/// Notice how the single quote was 'escaped' in the SQL way to
-	/// avoid a syntax error.
-	template <class Manip>
-	equal_list_ba<FieldNames, Row, Manip> equal_list(const char* d,
-			const char* e, Manip m) const;
-
 private:
-	std::vector<std::string> data_;
-	std::vector<bool> is_nulls_;
-	const ResUse* res_;
+	list_type data_;
+	RefCountedPointer<FieldNames> field_names_;
 	bool initialized_;
 };
 
 } // end namespace mysqlpp
 
-#endif
+#endif // !defined(MYSQLPP_ROW_H)
